// --- [owi_lib_s.c ] ------------------------------------------------------------
//	     tab = 3
//      basiert auf Maxim Application Note 127 / 187 / an187.zip
//
//      vereinfachte Version fr einen einzelnen Sensor,
//      erweitert auf das Einlesen einzelner Sensoren an mehreren Portpins.
//      Der Sensor wird ber SKIP_ROM angesprochen, sein Code wird nicht gespeichert
//		  Nach dem OWconvert muss irgendwie fr die Dauer von ca. 800ms gewartet werden
//      Danach kann der einzige an einem Pin angeschlossene Sensor ausgelesen werden.
//      Dazu wird DS18B20_read mit der Pinmaske fr genau einen auszulesenden Sensor
//      aufgerufen.

/*			Anwendung:

			OW_convert(Pinmaske);
			// Konversionszeit von 800ms verstreichen lassen
			(int16_t) temperatur = DS18B20_read(Pinmaske);

			Potentielle Kollisionen mit anderen Programmteilen: der PORT fr die DQ-Pins !!
*/

// ----------------------------------------------------------------------------
#define EXTERN

#include <avr/io.h>
#include <stdlib.h>
#include <inttypes.h>
#include <util/delay.h>
#include "global.h"
#include "owi_lib_sx.h"
#include "rs232_mini.h"

#define  SCRATCHPAD_SIZE		5				// Die Anzahl der Bytes, 
														// die aus dem Scratchpad gelesen werden
// --- lokale Variablen --------------------------------------------------------
uint8_t ow_buffer[SCRATCHPAD_SIZE];			// fr die Daten des Scratchpads
uint8_t ow_pin_mask;

/// --- lokale Funktionen ------------------------------------------------------
uint8_t OWReset(void);
void    OWWriteByte(unsigned char byte_value);
void    OWWriteBit(unsigned char bit_value);
uint8_t OWReadByte(void);
uint8_t OWReadBit(void);

// ----------------------------------------------------------------------------
// sendet ein SKIP ROM mit nachfolgendem CONVERT an alle mithrenden Sensoren
// Die Daten werden auf allen Pins ausgegeben, fr die in der Maske ein Bit gesetzt ist.
//
// Danach muss ca. 800 MilliSekunde gewartet werden, bis die konvertierten Daten aus
// den einzelnen Sensoren ausgelesen werden knnen.
void OW_convert(uint8_t mask)
{
ow_pin_mask = mask;								// wird zur Auswahl der Sensoren genutzt

OWReset();
OWWriteByte(SKIP_ROM);
OWWriteByte(CONVERT);
}

// ----------------------------------------------------------------------------
// rechnet den Messwert, der aus 2 Byte besteht, in ein int16_t um.
// Der Messwert steht rechtsbndig in den globalen Variablen ow_buffer[0] und ow_buffer[1],
// (die Werte entsprechen scratchpad.Byte[0] / scratchpad.Byte[1])
int16_t DS18B20_read(uint8_t mask)
{
uint8_t i;
int16_t temp16;

ow_pin_mask = mask;								// wird zur Auswahl der Sensoren genutzt

OWReset();											// ein Reset senden
OWWriteByte(SKIP_ROM);

OWWriteByte(READ_SCRATCHPAD);
														// liest nur die ersten 5 Byte aus dem Scratchpad aus
for (i = 0; i < SCRATCHPAD_SIZE; i++) ow_buffer[i] = OWReadByte();
OWReset();											// Keine weiteren Bytes einlesen !

														// nun steht der ROM-Code in ow_buffer
// Wenn kein Sensor angeschlossen ist, dann liefern alle Bytes den Wert 0xFF
// Das Byte 4 (Configuration Byte) hat aber immer den Wert 0x7F !!
// Wird dagegen 0xFF gelesen, dann ist der Sensor nicht vorhanden.
// In diesem Falle wird der Wert -99.9 zurckgegeben.
if (ow_buffer[4] & 0x80) return(-999);

temp16  = (ow_buffer[1] << 8);				// das high-Byte nach links in die 16bit-Variable schieben 
temp16 += (ow_buffer[0] & 0xF0);				// das low-Byte ohne Dezimalstellen addieren
temp16 /= 16;										// durch Division alles um 4 Stellen nach rechts schieben (dabei Sign-Bits einfgen !!) 
temp16 *= 10;										// und mit 10 multiplizieren, um Platz fr die Nachkommastelle zu machen

// die Nachkommastelle: temp_lo * 0.625 = temp_lo * 625 / 1000 = temp_lo * 640 / 1024
ow_buffer[0] &= 0x0F; 							// die Nachkommastelle freistellen 
temp16 += (int16_t)(ow_buffer[0]) * 640 / 1024;// von 16 Stellen auf 10 Stellen umrechnen

return(temp16);
}

//-----------------------------------------------------------------------------
// 1-Wire Functions to be implemented for a particular platform
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Reset the 1-Wire bus and return the presence of any device
// Return TRUE  : device present
//        FALSE : no device present
// alle Wartezeiten gem. APP_Note 126, Page 2-3 
// Gesamtdauer: DELAY_G + DELAY_H + DELAY_I + DELAY_J = 1 + 480 + 70 + 410 = 921us
uint8_t OWReset(void)
{
uint8_t r;

_delay_us(DELAY_G);

DQ_LOW;												// DQ auf Low ziehen
_delay_us(DELAY_H);
DQ_INPUT;											// DQ als Eingang schalten
DQ_HIGH;												// auf high schalten (Pullup aktivieren)
_delay_us(DELAY_I);								// warten bis zum Lesen der Antwort

r = DQ_READ;										// Antwort einlesen
if (r) r = FALSE; else r = TRUE;				// die Leitung sollte auf LOW gezogen sein
DQ_OUTPUT;											// DQ wieder als Ausgang schalten
_delay_us(DELAY_J);
	
return(r);
}

//-----------------------------------------------------------------------------
// Send 8 bits of data to the 1-Wire bus
//
void OWWriteByte(unsigned char byte_value)
{
uint8_t i = 8;

while(i--)
	{
	OWWriteBit(byte_value & 0x01);			// das rechts stehende Bit senden
	byte_value = byte_value >> 1;				// um 1 Stelle nach rechts schieben
	}
}

//-----------------------------------------------------------------------------
// Send 1 bit of data to the 1-Wire bus
// DG ist durchgngig als Ausgang geschaltet
// alle Wartezeiten gem. APP_Note 126, Page 2-3 
// Gesamtdauer: DELAY_A + DELAY_B = DELAY_C + DELAY_D = 6 + 64 = 60 + 10 = 70us
void OWWriteBit(uint8_t bit_value)
{
asm volatile ("cli");
DQ_LOW;												// DQ auf Low ziehen

if (bit_value)										// Write 1
	{
	_delay_us(DELAY_A);							// 5 us auf Low lassen
	DQ_INPUT;										// DQ durch Pullup auf High ziehen
	_delay_us(DELAY_B);							// 40us warten
	}
	
else													// Write 0
	{
	_delay_us(DELAY_C);							// 45us auf Low lassen
	DQ_INPUT;										// DQ durch Pullup auf High setzen
//	DQ_HIGH;
	_delay_us(DELAY_D);
	}

DQ_OUTPUT;											// DQ wieder als Ausgang schalten
DQ_HIGH;
asm volatile ("sei");
}

// ----------------------------------------------------------------------------
uint8_t OWReadByte(void)
{
uint8_t i = 8;
uint8_t data = 0;
while(i--)
	{
	data >>= 1;
	if (OWReadBit()) data |= 0x80;
	}
return(data);
}

//-----------------------------------------------------------------------------
// Read 1 bit of data from the 1-Wire bus
// Return 1 : bit read is 1
//        0 : bit read is 0
// alle Wartezeiten gem. APP_Note 126, Page 2-3 
// Gesamtdauer: DELAY_A + DELAY_E + DELAY_F = 6 + 9 + 55 = 70us
uint8_t OWReadBit(void)
{
uint8_t r;
asm volatile ("cli");
DQ_LOW;												// DQ auf Low ziehen
_delay_us(DELAY_A);								// 6 us warten
DQ_INPUT;											// als Eingang schalten
DQ_HIGH;

_delay_us(DELAY_E);								// warten und dann messen
r = DQ_READ;

_delay_us(DELAY_F);

DQ_OUTPUT;											// DQ wieder als Ausgang schalten

asm volatile ("sei");
return (r);

}

// --- [ eof ] ----------------------------------------------------------------